#!/usr/bin/perl
#
# ############
# Copyright 2005-2006, Jefff Hardy (hardyjm at potsdam dot edu)
# Copyright 2006, Neil Cherry (linuxha@linuxha.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ###########
#
#
# rename this script x10 and put it in the cups/backend/ directory
# On my system it's in /usr/lib/cups/backend/
# example URI - x10://parallel:/dev/lp0
#
# This backend is a Perl script designed to be run as a wrapper around other
# CUPS backends.  
#
# -[ History ]----------------------------------------------------------------
# 20060522 - I took Jeff's origianl Perl script, trimmed it, added a few
#            commands and called it my own. :-) Jeff get full credit for this.
# -[ Todo ]-------------------------------------------------------------------
# - Need to figure out a way to not turn off the printer if a second job comes
#   in. Right now the X10 module will be turned off 5 minutes after the job
#   has run unless there manual intervention (atrm <job_#>)
# ----------------------------------------------------------------------------
my @argv = @ARGV;					#keep hands off the argument array

my $DEBUG = 0;						#turn on/off [1/0] debugging function call

### ARGS CHECK
# If too many arguments, send error
if (scalar(@argv) < 5 || scalar(@argv) > 6){
        print LOG ("ERROR: Usage: accsnmp job-id user title copies options [file]\n");
	exit 1;
}

# If no arguments, device discovery mode
if (!$argv[0]){
	print ("network accsnmp \"Unknown\" \"Accounted Printer (SNMP)\"\n");
	exit 0;
}
# If too many arguments, send error
if (scalar(@argv) < 5 || scalar(@argv) > 6){
        print STDERR ("ERROR: Usage: accsnmp job-id user title copies options [file]\n");
	exit 1;
}

# Now that we got this far, let us name the arguments to keep our sanity
my ($jobID,$userName,$jobTitle,$copies,$printOptions,$printFile) = @argv;

### SPOOL JOB TO LOCAL FILE
# Establish a temp file to which to spool
my $jid = $jobID;
my $uid = $userName;
$jid =~ s/\W//g;	#sanity check
$uid =~ s/\W//g;	#sanity check
my $tempFile = "/var/tmp/$jid-$uid-cupsjob$$";

open (OUT, ">$tempFile") or die "ERROR: Cannot write $tempFile: $!\n";
# If filename argv5 was supplied, that is the source filename
if ($printFile){ 
        open (FH, "$printFile") or die "ERROR: Cannot read $printFile: $!\n";
	while(<FH>){
        	print OUT "$_";
	}
	close FH;
} else { # Else stdin is the source
	while(<STDIN>){
        	print OUT "$_";
	}
}
close OUT;


### PRINTING
# Pull out the pieces of the device uri from the environment
my @uriParts = split(/\/\//,$ENV{DEVICE_URI});

my $wrapBackend = $uriParts[0];		#this backend wrapper
$wrapBackend =~ s/://;
my $realBackend = $uriParts[1];		#the wrapped backend
$realBackend =~ s/://;
my $printerIP = $uriParts[2];		#printer ip for snmp
$printerIP =~ s/:.*//;

# NOTE: the next several lines are the guts of wrapping a cups backend
# 1) Fix device uri environment variable by removing this wrapper backend from the string.
# 2) Tack the fixed device uri onto the front of the arg array so it is passed onto the real backend.
# 3) Tack on our spool file to arg array if job arrived via stdin (no printfile).
# This will happen on non-raw queues and we need to check because if we do not it
# will dos the printer with a "zero byte" error.
# 4) Call the real backend.
$ENV{DEVICE_URI} =~ s#$wrapBackend://##;
unshift (@argv, $ENV{'DEVICE_URI'});
if (!($printFile)){
	@argv = (@argv,$tempFile);
}

# -[ Turn on the X10 Printer w/Heyu ]-----------------------------------------
# Using Heyu
`heyu on printer`;

#my exitval = system {"/usr/bin/heyu"} @argv;

print LOG ("System command = system \{/usr/lib/cups/backend/$realBackend\} @argv\n");
print LOG ("X10 Printer ON\n");

my $exitval = system {"/usr/lib/cups/backend/$realBackend"} @argv;
$exitval >>= 8;

# -[ Turn off the X10 Printer now+5 min ]-------------------------------------
# The reason you need to use the 'at' command is that if you send an off
# command immediately the printer will stop even if it has something in the
# buffer. Waiting 5 minutes is a safe bet. The buffer should clear and
# the printing be done even if you have an error and need a minute to
# clear it.
# Need to run the command 'at' to time the X10 off at now+5 minutes.
my file_path = "/usr/lib/cups/backend/x10off.sh";
if( -x $file_path ) {
    `at -f $file_path now + 5 minutes`;
} else {
    print "File: $file_path not found\n";
}

close (LOG);

### CLEANUP/EXIT

if (!$SAVE_JOBS){
	unlink $tempFile;
}
exit $exitval;
                                                                                                             
###################### END MAIN ######################


